require "import"
import "android.widget.*"
import "android.view.*"

-- Game State
local gameState = {
    players = {}, -- Player details
    currentPlayer = 1, -- Track whose turn it is
    totalSquares = 100, -- Total squares on the board
    gameStarted = false,
    skipTurns = {0, 0}, -- Track skipped turns for players
    lethalSquares = {13, 66, 88}, -- Squares with predefined events
    beneficialSquares = {5, 35, 70}, -- Squares with beneficial events
    difficultyLevel = 1, -- Difficulty level starts at 1
    savedData = {} -- Placeholder for saved player data
}

-- Event Definitions
local events = {
    {text = " discovered a shortcut! Advance by 20 squares!", move = 20, skip = 0},
    {text = " fell into a pit! Move back to square one!", move = -100, skip = 0},
    {text = " got lost! Skip your next turn.", move = 0, skip = 1},
    {text = " found a trampoline! Jump ahead by 30 squares!", move = 30, skip = 0},
    {text = " triggered a trap! Move back 10 squares.", move = -10, skip = 0},
    {text = " made a new friend. Advance by 10 squares!", move = 10, skip = 0},
    {text = " got scared by a ghost! Skip two turns!", move = 0, skip = 2},
    {text = " discovered a treasure chest! Gain 25 squares!", move = 25, skip = 0},
    {text = " stepped into quicksand. Move back 15 squares.", move = -15, skip = 0}
}

-- Event Probability
local eventChance = 25 -- Percentage chance for an event to occur

-- UI Layouts
local mainMenuLayout = {
    LinearLayout,
    orientation = "vertical",
    layout_width = "fill",
    layout_height = "fill",
    padding = "16dp",
    {
        TextView,
        text = "Welcome to the 100-Square Adventure Game!",
        textSize = "24sp",
        layout_width = "fill",
        layout_height = "wrap_content",
        gravity = "center"
    },
    {
        Button,
        text = "Start Game",
        layout_width = "fill",
        layout_height = "wrap_content",
        onClick = "openSignUpWindow"
    },
    {
        Button,
        text = "Restore Existing Player Account",
        layout_width = "fill",
        layout_height = "wrap_content",
        onClick = "restorePlayerData"
    }
}

local signUpLayout = {
    LinearLayout,
    orientation = "vertical",
    layout_width = "fill",
    layout_height = "fill",
    padding = "16dp",
    {
        EditText,
        id = "player1_name",
        hint = "Enter Player 1 Name",
        layout_width = "fill",
        layout_height = "wrap_content"
    },
    {
        EditText,
        id = "player2_name",
        hint = "Enter Player 2 Name",
        layout_width = "fill",
        layout_height = "wrap_content"
    },
    {
        Button,
        text = "Start Game",
        layout_width = "fill",
        layout_height = "wrap_content",
        onClick = "startGame"
    }
}

local gameLayout = {
    LinearLayout,
    orientation = "vertical",
    layout_width = "fill",
    layout_height = "fill",
    padding = "16dp",
    {
        TextView,
        id = "player_turn",
        text = "Game starts!",
        textSize = "20sp",
        layout_width = "fill",
        layout_height = "wrap_content"
    },
    {
        Button,
        text = "Roll Dice",
        layout_width = "fill",
        layout_height = "wrap_content",
        onClick = "rollDice"
    }
}

-- Open Sign-Up Screen
function openSignUpWindow()
    activity.setContentView(loadlayout(signUpLayout))
end

-- Restore Existing Player Data
function restorePlayerData()
    if gameState.savedData.player1 and gameState.savedData.player2 then
        player1_name.setText(gameState.savedData.player1)
        player2_name.setText(gameState.savedData.player2)
        Toast.makeText(activity, "Welcome back, " .. gameState.savedData.player1 .. " & " .. gameState.savedData.player2 .. "!", Toast.LENGTH_SHORT).show()
        startGame()
    else
        Toast.makeText(activity, "No saved data found! Please enter names manually.", Toast.LENGTH_SHORT).show()
        openSignUpWindow()
    end
end

-- Start Game
function startGame()
    local player1 = player1_name.getText().toString()
    local player2 = player2_name.getText().toString()

    if player1 == "" or player2 == "" then
        Toast.makeText(activity, "Enter both names!", Toast.LENGTH_SHORT).show()
        return
    end

    gameState.players = {
        {name = player1, position = 0},
        {name = player2, position = 0}
    }
    gameState.savedData = {
        player1 = player1,
        player2 = player2
    }
    gameState.gameStarted = true
    openGameScreen()
end

-- Open Game Screen
function openGameScreen()
    activity.setContentView(loadlayout(gameLayout))
    updateUI()
end

-- Roll Dice
function rollDice()
    if gameState.skipTurns[gameState.currentPlayer] > 0 then
        Toast.makeText(activity, gameState.players[gameState.currentPlayer].name ..
            " skips this turn!", Toast.LENGTH_SHORT).show()
        gameState.skipTurns[gameState.currentPlayer] = gameState.skipTurns[gameState.currentPlayer] - 1
        switchTurn()
        return
    end

    local currentPlayer = gameState.players[gameState.currentPlayer]
    local dice = math.random(1, 6)
    currentPlayer.position = math.min(currentPlayer.position + dice, gameState.totalSquares)

    -- Check for Winner
    if currentPlayer.position == gameState.totalSquares then
        Toast.makeText(activity, currentPlayer.name .. " wins the game!", Toast.LENGTH_LONG).show()
        return
    end

    -- Trigger Random Event
    local eventMessage, move, skip = triggerEvent(currentPlayer)
    if move then
        currentPlayer.position = math.min(gameState.totalSquares, math.max(0, currentPlayer.position + move))
    end
    if skip then
        gameState.skipTurns[gameState.currentPlayer] = skip
    end

    -- Show dice roll, event, and new position
    Toast.makeText(activity, currentPlayer.name .. " rolled a " .. dice .. ". " .. eventMessage ..
        " New position: " .. currentPlayer.position, Toast.LENGTH_LONG).show()

    -- Increase Difficulty Level
    increaseDifficulty()

    -- Switch Turn
    switchTurn()
end

-- Update UI
function updateUI()
    local currentPlayer = gameState.players[gameState.currentPlayer]
    player_turn.setText("It's " .. currentPlayer.name .. "'s turn. Position: " .. currentPlayer.position)
end

-- Switch Turn
function switchTurn()
    gameState.currentPlayer = gameState.currentPlayer % #gameState.players + 1
    updateUI()
end

-- Trigger Event
function triggerEvent(player)
    local square = player.position
    if math.random(1, 100) <= eventChance then
        local event = events[math.random(#events)]
        local eventText = player.name .. event.text
        return eventText, event.move, event.skip
    end

    -- Check for Special Squares (Lethal and Beneficial Squares)
    if table.contains(gameState.lethalSquares, square) then
        return "Oh no! You stepped on a lethal square and fell into a pit! You're back to square one!", -player.position, 0
    end

    if table.contains(gameState.beneficialSquares, square) then
        local eventMessages = {
            "You found a hidden treasure chest! Plus 25 squares ahead!",
            "You discovered a magical portal! Teleport forward 15 squares!",
            "A fairy grants you wings! Fly ahead 20 squares!",
            "You found a bag of gold! Your next turn is +10 squares!"
        }
        local message = eventMessages[math.random(#eventMessages)]
        return message, math.random(10, 30), 0
    end

    return player.name .. " continues without incident.", 0, 0
end

-- Increase Difficulty: Increase event chance based on player's progress
function increaseDifficulty()
    local currentPlayer = gameState.players[gameState.currentPlayer]
    if currentPlayer.position > 50 then
        gameState.difficultyLevel = 2  -- Increase difficulty after position > 50
        eventChance = 35  -- Increase event chance to 35%
    elseif currentPlayer.position > 75 then
        gameState.difficultyLevel = 3  -- Further increase difficulty after position > 75
        eventChance = 45  -- Increase event chance to 45%
    end
end

-- Check if table contains value
function table.contains(tbl, val)
    for _, value in ipairs(tbl) do
        if value == val then
            return true
        end
    end
    return false
end

-- Start Game on Launch
activity.setContentView(loadlayout(mainMenuLayout))